#!/bin/sh

## Define function
# runiraf (iraf command) (output logfile) (output commandlist)
runiraf(){
  TMPLOG=$TMPPATH/tmp.log

  cat $1 >> $3
  cat $1 | cl > $TMPLOG
  cat $TMPLOG >> $2
  echo "" >> $2
  ERRSTAT=`grep ERROR $TMPLOG | gawk 'END{print(NR)}'`

  if [ $ERRSTAT -ne 0 ]; then
   echo "# Error in IRAF ... Exiting #"
   echo "# Check $2 #" 
   if [ $WFCAMDEBUG -eq 1 ]; then 
     \rm -r $TMPPATH
   fi
   exit 1
  fi
}

#Check Environmental Parameters
if [ `env | grep WFCAMDATA |gawk 'END{print(NR)}'` -eq 0 ]; then
 echo "Error : Environmental parameter WFCAMDATA not defined!!"
 exit 1
fi

if [ `env | grep WFCAMDATE |gawk 'END{print(NR)}'` -eq 0 ]; then
 echo "Error : Environmental parameter WFCAMDATE not defined!!"
 exit 1
fi

if [ `env | grep WFCAMLOG |gawk 'END{print(NR)}'` -eq 0 ]; then
 echo "Error : Environmental parameter WFCAMLOG not defined!!"
 exit 1
fi

# Debug Parameter 
# WFCAMDEBUG=1; erase all temporary files 
#            0; leave all temporary files 
if [ `env | grep AUTOMKDEBUG |gawk 'END{print(NR)}'` -eq 0 ]; then
WFCAMDEBUG=1
fi

#Check command arguments
if [ $# != 4 ]; then
    echo "Usage : "$0" (Header Name) (Detector ID) (Start No.) (No. of Frames)"
    exit 1
fi

HEAD=$1
DETECTOR=$2
IS=$3
N=$4

# Set Temporary directory
TMPPATHBASE=/tmp/$USER
if [ ! -d $TMPPATHBASE ];then
  mkdir $TMPPATHBASE
fi

COMMANDHEAD=obj.$HEAD.`date +%y%m%d%H%M%S`

TMPPATH=$TMPPATHBASE/$COMMANDHEAD
if [ ! -d $TMPPATH ];then
 mkdir $TMPPATH
fi

# setup filenames
CLCOM=$TMPPATH/cl.com
CLLOG=log/obj.$HEAD.`date +%y%m%d%H%M%S`.log
CLLOGCOM=`echo $CLLOG | sed 's/.log/.cl/g'`
TMPLOG=$TMPPATH/tmp.log

echo "$0 $1 $2 $3 $4"
echo "# $0 $1 $2 $3 $4" > $CLLOG

echo "# Log will be saved as "$CLLOG
echo "# IRAF commands will be saved as "$CLLOGCOM

# write environment parameters set to logfile
echo WFCAMDATA=$WFCAMDATA >> $CLLOG
echo WFCAMDATE=$WFCAMDATE >> $CLLOG
echo WFCAMLOG=$WFCAMLOG >> $CLLOG
echo WFCAMDEBUG=$WFCAMDEBUG >> $CLLOG

# Check Architecture
ARCH=`uname`
echo "# Running on "`uname -a`
echo "# Running on "`uname -a` >> $CLLOG

# Check IRAF Version
echo "logout" | cl 1> $TMPLOG
IRAFVER=`grep Revision $TMPLOG | sed 's/-/ /g' |  gawk '{split($0,a,"Revision");
print(a[2])}'| gawk '{split($1,a,".");print(a[1]+a[2]/100+a[3]/1000)}'`
echo "# IRAF Version $IRAFVER "
echo "# IRAF Version $IRAFVER " >> $CLLOG

#Start time
if [ $ARCH = "Linux" ]; then
    SECSTART=`date +%s` 
else 
    SECSTART=`date +"%d %H %M %S"| gawk '{print($1*24*3600+$2*3600+$3*60+$4)}'` 
fi
echo "# Start at $SECSTART" >> $CLLOG

# set filenames
SKYFLAT=flat.fits
DARK=dark${DETECTOR}.fits
BADPIX=badpix${DETECTOR}.fits
FINAL=${HEAD}_${DETECTOR}.fits

DATAPATH=$WFCAMDATA/wfcam${DETECTOR}/${WFCAMDATE}

TDATAPATH=./tdata
LDATAPATH=./ldata

if [ ! -d $LDATAPATH ];then
 mkdir $LDATAPATH
fi

if [ ! -d $TDATAPATH ];then
 mkdir $TDATAPATH
fi

# set detector ID specific information
case $DETECTOR in 
1) DETECTORC=w ; gawk -v is=$IS -v n=$N 'BEGIN{flag=0}{if($1==is){flag=1;i=is}if(flag>0 && $1==i){sub("/","",$10);print($11/0.2,-$10/0.2);i++}if(i>=is+n)exit}' $WFCAMLOG > offset.tbl ;;
2) DETECTORC=x ; gawk -v is=$IS -v n=$N 'BEGIN{flag=0}{if($1==is){flag=1;i=is}if(flag>0 && $1==i){sub("/","",$10);print(-$10/0.2,-$11/0.2);i++}if(i>=is+n)exit}' $WFCAMLOG > offset.tbl ;;
3) DETECTORC=y ; gawk -v is=$IS -v n=$N 'BEGIN{flag=0}{if($1==is){flag=1;i=is}if(flag>0 && $1==i){sub("/","",$10);print(-$11/0.2,$10/0.2);i++}if(i>=is+n)exit}' $WFCAMLOG > offset.tbl ;;
4) DETECTORC=z ; gawk -v is=$IS -v n=$N 'BEGIN{flag=0}{if($1==is){flag=1;i=is}if(flag>0 && $1==i){sub("/","",$10);print($10/0.2,$11/0.2);i++}if(i>=is+n)exit}' $WFCAMLOG > offset.tbl ;;
esac

# create local file table
gawk -v path=$DATAPATH -v date=$WFCAMDATE -v is=$IS -v n=$N -v detector=$DETECTORC 'BEGIN{if(n>0){for(i=is;i<is+n;i++)printf("%s/%s%s_%05d.fit\n",path,detector,date,i)}}' > data.tbl
gawk -v path=$TDATAPATH '{n=split($0,a,"/");print(path"/"a[n])}' data.tbl | sed 's/.fit$/.fits/g' > tdata.tbl
gawk -v path=$LDATAPATH '{n=split($0,a,"/");print(path"/"a[n])}' data.tbl | sed 's/.fit$/.fits/g' > ldata.tbl
gawk -v path=$LDATAPATH '{n=split($0,a,"/");print(path"/"a[n])}' data.tbl | sed 's/.fit$/.fits[513:1536,513:1536]/g' > ldata-1.tbl

echo "## Copy data to local disk with dark subtraction and flat-fielding"
echo "## Copy data to local disk with dark subtraction and flat-fielding" >> $CLLOG

cat << EOF > $CLCOM
 imdel @ldata.tbl
 imdel @tdata.tbl >> $CLCOM
 logout
EOF
runiraf $CLCOM $CLLOG $CLLOGCOM 

paste data.tbl ldata.tbl | gawk -v flat=$SKYFLAT -v dark=$DARK '{printf("./bin/fwfcam %s\"[1]\" %s %s %s\n",$1,$2,dark,flat)}' | csh 1>> $CLLOG

echo "## Fix Header Information"
echo "## Fix Header Information" >> $CLLOG
gawk '{printf("hedit %s ctype1 RA ver-\nhedit %s ctype2 DEC ver-\n",$1,$1)}'  ldata.tbl > $CLCOM

# 1st imstat
cat << EOF >> $CLCOM
 del ${TMPPATH}/tmp.dat
 imstati @ldata-1.tbl format- > ${TMPPATH}/tmp.dat
 logout
EOF
runiraf $CLCOM $CLLOG $CLLOGCOM

# 2nd imstat
gawk -v tmppath=$TMPPATH 'BEGIN{printf("del %s/tmp.dat\n",tmppath)}{printf("imstati %s upper=%f lower=%f format- >> %s/tmp.dat\n",$1,$3+3*$4,$3-3*$4,tmppath)}' $TMPPATH/tmp.dat > $CLCOM
echo logout >>$CLCOM
runiraf $CLCOM $CLLOG $CLLOGCOM

echo "## Uniform background level"
echo "## Uniform Background level" >> $CLLOG
gawk '{s+=$3;a[NR]=$3}END{s=s/NR;for(i=1;i<=NR;i++)print(a[i]-s)}' ${TMPPATH}/tmp.dat > ${TMPPATH}/tmp2.dat

cat << EOF > $CLCOM
 imarith @ldata.tbl - @${TMPPATH}/tmp2.dat @ldata.tbl
 logout
EOF
runiraf $CLCOM $CLLOG $CLLOGCOM 

echo "## Divide pixels into 2x2"
echo "## Divide pixels into 2x2" >> $CLLOG
if [ -f $BADPIX ]; then
cat <<EOF > $CLCOM
imdel ${TMPPATH}/tmpbadpix.pl
magnify $BADPIX ${TMPPATH}/tmpbadpix.pl 2 2 fluxcon-
logout
EOF
runiraf $CLCOM $CLLOG $CLLOGCOM
fi

cat <<EOF > $CLCOM
imdel @tdata.tbl
magnify @ldata.tbl @tdata.tbl 2 2 interpo=nearest
EOF

# add badpixel mask data to header
if [ -f $BADPIX ]; then
for tdata in `cat tdata.tbl`;do
 echo hedit $tdata bpm ${TMPPATH}/tmpbadpix.pl add+ ver- >> $CLCOM
done
fi

echo logout >> $CLCOM
runiraf $CLCOM $CLLOG $CLLOGCOM

echo "## Shift and Add all frames"
echo "## Shift and Add all frames" >> $CLLOG

cat <<EOF > $CLCOM
imdel $FINAL
 imcombine @tdata.tbl $FINAL combine=average reject=sigclip offsets=offset.tbl masktype=goodvalue maskvalue=0
 disp $FINAL 2 fi+
 logout
EOF
runiraf $CLCOM $CLLOG $CLLOGCOM

echo "# Final frame saved as $FINAL"

#END time
if [ $ARCH = "Linux" ] ; then
    SECEND=`date +%s` 
else
    SECEND=`date +"%d %H %M %S"| gawk '{print($1*24*3600+$2*3600+$3*60+$4)}'` 
fi

echo "# End at $SECEND" >> $CLLOG
echo $SECSTART $SECEND | gawk '{printf("# Took %d sec to complete\n",$2-$1)}'
echo $SECSTART $SECEND | gawk '{printf("# Took %d sec to complete\n",$2-$1)}' >> $CLLOG

echo "# Log saved as $CLLOG"
echo "# IRAF commands saved as $CLLOGCOM"
if [ $WFCAMDEBUG -eq 1 ]; then 
  \rm -r $TMPPATH
else 
  echo "# See $TMPPATH for temporary files"
fi

exit 0




